//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.web;

import java.io.IOException;
import java.io.Serializable;
import java.util.Collection;
import java.util.PropertyResourceBundle;
import java.text.MessageFormat;

import javax.ejb.FinderException;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.bank.ejb.AccountKey;
import com.ibm.websphere.samples.bank.ejb.AccountLocal;
import com.ibm.websphere.samples.bank.ejb.AccountLocalHome;
import com.ibm.websphere.samples.bank.ejb.Customer;
import com.ibm.websphere.samples.bank.ejb.CustomerHome;
import com.ibm.websphere.samples.bank.ejb.CustomerKey;

import java.util.logging.*;

/**
 * This servlet searches for a list of accounts using the customer number and 
 * the balance exceeding a certain amount. Balance is not a required field. The 
 * default is zero.
 *
 *
 * This servlet is also used for the user-friendly navigation from result set 
 * of the search by customer module. In the list of customers page the customer 
 * number has a link that displays the list of accounts. To display the list of 
 * accounts this servlet is used to get the result set.
 */

public class SearchByAccountServlet extends HttpServlet implements Serializable {

    private CustomerHome customerHome = null;
    private AccountLocalHome accountHome = null;
    private final static String CUSTOMERHOME_JNDI_NAME = "java:comp/env/ejb/CustomerHome";

    private static final String PROPFILE = "/WEB-INF/properties/bank_messages";
    PropertyResourceBundle bundle = (PropertyResourceBundle) PropertyResourceBundle.getBundle(PROPFILE);
    private final static String ACCOUNT_JNDI_NAME = "java:comp/env/ejb/Account";

    static final String className = SearchByAccountServlet.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    /**
    * Initialize the servlet
    */
    public void init(ServletConfig config) throws ServletException {
        final String methodName = "init";
        try {
            messageLogger = Logger.getLogger(className, rbName);
        } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        super.init(config);

        try {
            javax.naming.Context ctx = new javax.naming.InitialContext();
            Object obj = ctx.lookup(CUSTOMERHOME_JNDI_NAME);
            customerHome = (CustomerHome) javax.rmi.PortableRemoteObject.narrow(obj, CustomerHome.class);

            accountHome = (AccountLocalHome) ctx.lookup(ACCOUNT_JNDI_NAME);
        } catch (NamingException e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errNamingInit") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInitExpection") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }
    }

    /**
    * doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);
    }

    /**
    * doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);

    }

    /**
     * @param req
     * @param resp
     * @throws ServletException
     */
    public void performTask(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        final String methodName = "performTask";

        try {
            PageBean pgBean = new PageBean();
            java.util.ArrayList msg = new java.util.ArrayList();
            java.util.ArrayList errmsg = new java.util.ArrayList();

            String strCustNum = req.getParameter("customerNum");
            String strBalance = req.getParameter("balance");

            long longCustNum = 0;
            float floatBalance = 0;

            if (strCustNum == null || strCustNum.trim().length() == 0) {
                errmsg.add(bundle.getString("errCust"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errCust"));
            } else {
                try {
                    longCustNum = Long.parseLong(strCustNum);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errCustFormat"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errCustFormat"));
                }
            }
            if (strBalance == null || strBalance.trim().length() == 0) {
                //default to zero	
                strBalance = "0";
                errmsg.add(bundle.getString("errBalanceExcc"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errBalanceExcc"));
            } else {
                try {
                    floatBalance = Long.parseLong(strBalance);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errBalanceEccFormat"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errBalanceEccFormat"));
                }
            }
            java.util.ArrayList list = new java.util.ArrayList();

            //if no errors are found
            if (errmsg.size() == 0) {
                try {
                    CustomerKey cKey = new CustomerKey(longCustNum);

                    Customer customer = customerHome.findByPrimaryKey(cKey);
                    Collection col = customer.getAccountsByBalance(floatBalance);

                    java.util.Iterator it = col.iterator();

                    while (it.hasNext()) {
                        Integer a = (Integer) it.next();

                        AccountKey akey = new AccountKey(a.intValue());
                        AccountLocal acc = accountHome.findByPrimaryKey(akey);
                        AccountData data = new AccountData(acc.getAccountNumber(), acc.getBalance(), acc.getAccountType());

                        list.add(data);
                    }
                    pgBean.setAccountsByBalance(list);

                    Object[] msgArgs = new Object[2];
                    msgArgs[0] = strBalance;
                    msgArgs[1] = new Integer(list.size()).toString();
                    MessageFormat msgFormat = new MessageFormat(bundle.getString("succSearchByAcct"));
                    String msgAccts = msgFormat.format(msgArgs);
                    msg.add(msgAccts);
                    messageLogger.logp(Level.INFO, className, methodName, msgAccts);

                } catch (FinderException e) {
                    errmsg.add(bundle.getString("succNoAcct"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("succNoAcct") + e.getMessage());
                } catch (Exception e) {
                    errmsg.add(bundle.getString("errGenFailureAccount"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errGenFailureAccount") + e.getMessage());
                    e.printStackTrace();
                }
            } //end of if	

            if (pgBean.getAccountsByBalance() == null || pgBean.getAccountsByBalance().size() == 0) {
                msg.add(bundle.getString("succNoAcct"));
            }

            pgBean.setErrormessage(errmsg);
            pgBean.setMessageLines(msg);
            req.setAttribute("PageBean", pgBean);

            if (pgBean.getErrormessage().size() == 0 && pgBean.getAccountsByBalance().size() > 0) {
                getServletContext().getRequestDispatcher("/jsp/listofaccounts.jsp").forward(req, resp);
            } else {
                getServletContext().getRequestDispatcher("/jsp/searchbyaccount.jsp").forward(req, resp);
            }
        } catch (Throwable theException) {
            // uncomment the following line when unexpected exceptions are occuring to aid in debugging the problem
            // theException.printStackTrace();
            throw new ServletException(theException);
        }
    }
    /**
     * @param x
     * @return String representation of float
     */
    private String formatFloat(float x) {
        String s = null;
        if (((x * 100) % 10) == 0) {
            s = x + "0";
        } else {
            s = String.valueOf(x);
        }
        return s;
    }
}
