//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.web;

import java.io.IOException;
import java.io.Serializable;
import java.util.PropertyResourceBundle;
import java.text.MessageFormat;

import javax.ejb.FinderException;
import javax.naming.NamingException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.ibm.websphere.samples.bank.ejb.TransferLocal;
import com.ibm.websphere.samples.bank.ejb.TransferLocalHome;
import com.ibm.websphere.samples.bank.exception.InsufficientFundsException;

import java.util.logging.*;

/**
 * This servlet transfers funds from one account to another account. If the 
 * amount is not sufficient for a transfer it throws an exception.
 *
 */
public class TransferFundsServlet extends HttpServlet implements Serializable {

    // EJB access.
    private TransferLocalHome transferHome = null;
    private static final String PROPFILE = "/WEB-INF/properties/bank_messages";
    PropertyResourceBundle bundle = (PropertyResourceBundle) PropertyResourceBundle.getBundle(PROPFILE);
    private final static String TRANSFER_JNDI_NAME = "java:comp/env/ejb/Transfer";

    static final String className = TransferFundsServlet.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    /**
    * Initialize the servlet
    */
    public void init(ServletConfig config) throws ServletException {
        final String methodName = "init";
        try {
            messageLogger = Logger.getLogger(className, rbName);
       } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        super.init(config);

        try {
            javax.naming.Context ctx = new javax.naming.InitialContext();
            transferHome = (TransferLocalHome) ctx.lookup(TRANSFER_JNDI_NAME);
        } catch (NamingException e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errNamingInit") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errInitExpection") + e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }
    }
    /**
    * doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);
    }

    /**
    * doPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
    */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        performTask(req, resp);

    }

    /**
     * @param req
     * @param resp
     * @throws ServletException
     */
    public void performTask(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        final String methodName = "performTask";

        try {
            PageBean pgBean = new PageBean();
            java.util.ArrayList msg = new java.util.ArrayList();
            java.util.ArrayList errmsg = new java.util.ArrayList();

            java.text.DecimalFormat dF = new java.text.DecimalFormat("####.00");

            String fromBalance;
            String toBalance;

            javax.servlet.http.HttpSession session = req.getSession(true);
            HistoryBean history = (HistoryBean) session.getAttribute("HistoryBean"); 
            if (history == null) {
      	        history = new HistoryBean(); // creates the bean if there isn't one on the session
            }
            String strFromAcct = req.getParameter("fromAccount");
            String strToAcct = req.getParameter("toAccount");
            String strAmt = req.getParameter("amount");

            int intFromAcct = 0;
            int intToAcct = 0;
            float floatAmt = 0;

            if (strFromAcct == null || strFromAcct.trim().length() == 0) {
                errmsg.add(bundle.getString("errfromacct"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errfromacct"));
            } else {
                try {
                    pgBean.setFromAccount(Integer.parseInt(strFromAcct));
                    intFromAcct = Integer.parseInt(strFromAcct);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errfromacctValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errfromacctValid"));
                }
            }

            if (strToAcct == null || strToAcct.trim().length() == 0) {
                errmsg.add(bundle.getString("errtoacct"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errtoacct"));
            } else {
                try {
                    pgBean.setToAccount(Integer.parseInt(strToAcct));
                    intToAcct = Integer.parseInt(strToAcct);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("errtoacctValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errtoacctValid"));
                }
            }

            if (strAmt == null || strAmt.trim().length() == 0) {
                errmsg.add(bundle.getString("erramt"));
                messageLogger.logp(Level.INFO, className, methodName, bundle.getString("erramt"));
            } else {
                try {
                    pgBean.setTransAmt(Float.parseFloat(strAmt));
                    floatAmt = Float.parseFloat(strAmt);
                } catch (Exception ne) {
                    errmsg.add(bundle.getString("erramtValid"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("erramtValid"));
                }
            }

            //After page validation if there are no errors 
            if (errmsg.size() == 0) {
                try {
                    TransferLocal transfer = transferHome.create();
                    transfer.transferFunds(intFromAcct, intToAcct, floatAmt);

                    fromBalance = dF.format(transfer.getBalance(intFromAcct));
                    toBalance = dF.format(transfer.getBalance(intToAcct));

                    msg.add(bundle.getString("succTransfer"));
                    Object[] msgArgs = new Object[2];
                    msgArgs[0] = strFromAcct;
                    msgArgs[1] = fromBalance;
                    MessageFormat msgFormat = new MessageFormat(bundle.getString("succfromacct"));
                    String msgFromAccount = msgFormat.format(msgArgs);
                    msg.add(msgFromAccount);

                    msgArgs[0] = strToAcct;
                    msgArgs[1] = toBalance;
                    msgFormat = new MessageFormat(bundle.getString("succtoacct"));
                    String msgToAccount = msgFormat.format(msgArgs);
                    msg.add(msgToAccount);

                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("succTransfer"));
                    messageLogger.logp(Level.INFO, className, methodName, msgFromAccount);
                    messageLogger.logp(Level.INFO, className, methodName, msgToAccount);

                    history.addMessage(bundle.getString("succTransfer")); // adds a message to the command history
                    history.addMessage(msgFromAccount); // adds a message to the command history
                    history.addMessage(msgToAccount); // adds a message to the command history
                } catch (InsufficientFundsException e) {
                    errmsg.add(bundle.getString("errinsuff"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errinsuff"));
                } catch (FinderException e) {
                    errmsg.add(bundle.getString("errfindtransfer"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errfindtransfer"));
                } catch (Exception e) {
                    errmsg.add(bundle.getString("errtransExp"));
                    messageLogger.logp(Level.INFO, className, methodName, bundle.getString("errtransExp"));
                    e.printStackTrace();
                }
            } //end of if

            pgBean.setMessageLines(msg);
            pgBean.setErrormessage(errmsg);

            req.setAttribute("PageBean", pgBean);
            session.setAttribute("HistoryBean", history);// stores it back into the session

            getServletContext().getRequestDispatcher("/jsp/transferfunds.jsp").forward(req, resp);

        } catch (Throwable ex) {
            throw new ServletException(ex);
        }

    }
    /**
     * @param x
     * @return String representation of float
     */
    private String formatFloat(float x) {
        String s = null;
        if (((x * 100) % 10) == 0) {
            s = x + "0";
        } else {
            s = String.valueOf(x);
        }
        return s;
    }
}
