//
// "This sample program is provided AS IS and may be used, executed, copied and modified without royalty payment by customer (a) for its own 
// instruction and study, (b) in order to develop applications designed to run with an IBM WebSphere product, either for customer's own internal use 
// or for redistribution by customer, as part of such an application, in customer's own products. " 
//
// Product 5724-I63,  (C) COPYRIGHT International Business Machines Corp., 2004
// All Rights Reserved * Licensed Materials - Property of IBM
//
package com.ibm.websphere.samples.bank.util;


import javax.naming.*;

import com.ibm.websphere.samples.bank.ejb.*;
import java.util.*;

import java.util.logging.*;

public class InvokeEJB {
    private CustomerHome customerHome = null;
    private Context ctx = null;
    private final static String CUSTOMER_JNDI_NAME = "java:comp/env/ejb/Customer";

    static final String className = InvokeEJB.class.getName();
    private Logger messageLogger;
    private static final String rbName = null; // No localization is used for logging.

    public InvokeEJB() {
        final String methodName = "InvokeEJB";
        try {
            messageLogger = Logger.getLogger(className, rbName);
        } catch (Exception e) {
            // Problem initializing logging.  Continue anyway.
        }

        try {
            ctx = new InitialContext();
            init();
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, "Exception caught in constructor invokeEJB");
            e.printStackTrace();
        }

    }

    private void init() {
        final String methodName = "init";

        try {
            if (ctx == null)
                ctx = new InitialContext();

            Object localObject = ctx.lookup(CUSTOMER_JNDI_NAME);
            customerHome = (CustomerHome) javax.rmi.PortableRemoteObject.narrow(localObject, CustomerHome.class);
        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, "Exception caught in init invokeEJB");
            e.printStackTrace();
        }

    }

    /**
     *  find the customer with the specified search name ordered by last name
     * @param iname
     * @return Collection
     */
    public Collection findSortedCustomer(String iname) {
        final String methodName = "findSortedCustomer";

        ArrayList list = null;
        try {

            Collection col = customerHome.findByLastName(iname);
            Customer customer = null;
            Iterator it = col.iterator();

            while (it.hasNext()) {
                if (list == null)
                    list = new ArrayList();

                customer = (Customer) it.next();
                String fname = customer.getLastName();
                String lname = customer.getFirstName();

                long custnum = customer.getPrimaryKey().hashCode();
                CustomerData data = new CustomerData(custnum, fname, lname, customer.getTaxID());
                list.add(data);
            }

        } catch (Exception e) {
            messageLogger.logp(Level.INFO, className, methodName, "Exception :");
            e.printStackTrace();

        }
        return list;
    }

    /**
     * get the list of account numbers owned by customer
     * @param customerNumber
     * @return java.util.Collection collection of accountNumber (type long) 
     */
    public Collection getOwnedAccountsNumber(long customerNumber) {
        final String methodName = "getOwnedAccountsNumber";

        Collection accountNumbers = null;
        try {
            CustomerKey cKey = new CustomerKey(customerNumber);

            Customer customer = customerHome.findByPrimaryKey(cKey);
            accountNumbers = customer.getOwnedAccountNumbers();

            messageLogger.logp(Level.INFO, className, methodName, "Finding the customer owned account numbers");
            Iterator it = accountNumbers.iterator();

            while (it.hasNext()) {
                messageLogger.logp(Level.INFO, className, methodName, "Account Number: " + ((Long) it.next()).longValue());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return accountNumbers;
    }

    /**
     * get the list of account object own by the given customer
     * @param customerNumber
     * @return java.util.Collection collection of AccountData
     */
    public Collection getOwnedAccounts(long customerNumber) {
        final String methodName = "getOwnedAccounts";

        Collection accounts = null;
        try {
            CustomerKey cKey = new CustomerKey(customerNumber);

            Customer customer = customerHome.findByPrimaryKey(cKey);
            accounts = customer.getAccountsList();

            messageLogger.logp(Level.INFO, className, methodName, "Finding the customer owned account numbers");
            if (accounts != null) {
                Iterator it = accounts.iterator();

                while (it.hasNext()) {
                    AccountData act = (AccountData) it.next();
                    messageLogger.logp(Level.INFO, className, methodName, "Account Number: " + act.getAccNumber());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return accounts;
    }

    /**
     * get the number of accounts own by the given customer
     * this is the sample to illustrate the use of aggregate function COUNT
     * @param customerNumber
     * @return int
     */
    public int getAccountsCount(long customerNumber) {
        int count = 0;
        try {
            CustomerKey cKey = new CustomerKey(customerNumber);

            Customer customer = customerHome.findByPrimaryKey(cKey);
            count = customer.getAccountsCount();

        } catch (Exception e) {
            e.printStackTrace();
        }
        return count;
    }

    /**
     * get the list accounts number for a given customer that exceed the specified
     * balance
     * this is the sample to illustrate the use of EJB QL 
     * @param customerNumber
     * @param minBal
     * @return java.util.Collection collection of account number (type long)
     */
    public Collection getAccountsExceedBal(long customerNumber, float minBal) {
        Collection c = null;
        try {
            CustomerKey cKey = new CustomerKey(customerNumber);

            Customer customer = customerHome.findByPrimaryKey(cKey);
            c = customer.getAccountsByBalance(minBal);

        } catch (Exception e) {
            e.printStackTrace();
        }
        return c;
    }
}
